/* Copyright (c) 1993 by Sanjay Ghemawat */
#include <stdio.h>
#include <stdlib.h>
#include "Array.h"

void ArrayError::bounds(int b) {
    fprintf(stderr, "Array: %d out of bounds\n", b);
    abort();
}

void ArrayError::range(int r) {
    fprintf(stderr, "Array: %d out of range\n", r);
    abort();
}
