/* Copyright (c) 1993 by Sanjay Ghemawat */
#ifndef _MISC_TCL_H
#define _MISC_TCL_H

/*
 * Miscellaneous TCL support.
 */

#define TCL_Return(tcl,str) do { Tcl_SetResult(tcl, (str), TCL_STATIC); return TCL_OK; } while (0)

#define TCL_Error(tcl,str) do {Tcl_SetResult(tcl, (str), TCL_STATIC); return TCL_ERROR; } while (0)

#endif /* _MISC_TCL_H */
