/* Copyright (c) 1993 by Sanjay Ghemawat */
#ifndef _COMMANDS_H
#define _COMMANDS_H

/*
 * Prototype for Tcl commands.
 */

#include <tcl.h>

extern int Cmd_CreateCalendar	(ClientData, Tcl_Interp*, int, char*[]);
extern int Cmd_CreateNotice	(ClientData, Tcl_Interp*, int, char*[]);
extern int Cmd_CreateAppt	(ClientData, Tcl_Interp*, int, char*[]);

extern int Cmd_CharClass	(ClientData, Tcl_Interp*, int, char*[]);

extern int Cmd_Date		(ClientData, Tcl_Interp*, int, char*[]);
extern int Cmd_Time		(ClientData, Tcl_Interp*, int, char*[]);

extern int Cmd_PsMonth		(ClientData, Tcl_Interp*, int, char*[]);
extern int Cmd_PsDays		(ClientData, Tcl_Interp*, int, char*[]);

/*
 * Speedup routines.
 */
extern int Cmd_MonthDays	(ClientData, Tcl_Interp*, int, char*[]);
extern int Cmd_InterestTags	(ClientData, Tcl_Interp*, int, char*[]);

/*
 * Window routines.
 */
extern int Cmd_WindowRaise	(ClientData, Tcl_Interp*, int, char*[]);
extern int Cmd_Beep		(ClientData, Tcl_Interp*, int, char*[]);

#endif /* _COMMANDS_H */
