/* Copyright (c) 1993 by Sanjay Ghemawat */
#ifndef _HASH_H
#define _HASH_H

/*
 * Common hash functions.
 */

extern unsigned int hash_int(int);
extern unsigned int hash_string(char const*);

#endif /* _HASH_H */
